==> apigw-1.0.3.tgz: package/src/api.ts <==
import {
  CamelCasedProps,
  pascalCaseProperty,
  removeUndefined,
} from '@tencent-serverless-framework/common';
import {
  ApiInfo,
  CreateApiRequest,
  DesApisStatus,
  ModifyApiRequest,
} from 'tencentcloud-sdk-nodejs/tencentcloud/services/apigateway/v20180808/apigateway_models';
import { Apigw } from '.';
import { marshalApiInput } from './formatter';
import {
  ApiDeployerInputs,
  ApiDeployerOutputs,
  ApigwApiRemoverInputs,
  CreateOrUpdateApiInputs,
} from './interface';

function cleanReq(req: any) {
  delete req.EnableCors;
  delete req.Created;
  return req;
}

/** API 网关组件 API 子模块 */
export class Api {
  /** API 网关组件 */
  apigw: Apigw;
  /**
   * 构造函数
   *
   * @param apigw API 网关组件（依赖注入）
   */
  constructor(apigw: Apigw) {
    this.apigw = apigw;
  }

  /**
   * 部署 API 网关 API列表
   *
   * @param inputs 部署 API 网关输入
   * @returns 部署 API 网关输出
   */
  async apiDeployer(inputs: ApiDeployerInputs): Promise<ApiDeployerOutputs> {
    const { serviceId, environment, apiList = [], oldList, apiConfig, isOauthApi } = inputs;
    // if exist in state list, set created to be true
    const [exist] = oldList.filter(
      (item) =>
        item.method.toLowerCase() === apiConfig.method.toLowerCase() &&
        item.path === apiConfig.path,
    );

    if (exist) {
      apiConfig.apiId = exist.apiId;
      apiConfig.created = exist.created;

      if (isOauthApi) {
        apiConfig.authRelationApiId = exist.authRelationApiId;
      }
    }
    if (isOauthApi && !apiConfig.authRelationApiId) {
      // find reletive oauth api
      const { authRelationApi } = apiConfig;
      if (authRelationApi) {
        const [relativeApi] = apiList.filter(
          (item) =>
            item.method.toLowerCase() === authRelationApi.method.toLowerCase() &&
            item.path === authRelationApi.path,
        );
        if (relativeApi) {
          apiConfig.authRelationApiId = relativeApi.apiId;
        }
      }
    }

    const curApi = await this.createOrUpdateApi({
      serviceId,
      environment,
      endpoint: apiConfig,
      created: exist && exist.created,
    });

    console.log(`Deploy api ${curApi.apiName} success`);
    return curApi;
  }

  /**
   * 移除 API 网关 API
   *
   * @param inputs 移除 API 网关 API输入参数
   */
  async apiRemover(inputs: ApigwApiRemoverInputs) {
    const { apiConfig, serviceId, environment } = inputs;
    // 1. remove usage plan
    if (apiConfig.usagePlan) {
      await this.apigw.usagePlan.removeOrUnbindUsagePlan({
        serviceId,
        environment,
        apiId: apiConfig.apiId,
        usagePlan: apiConfig.usagePlan,
      });
    }

    // 2. delete only apis created by serverless framework
    if (apiConfig.apiId && apiConfig.created === true) {
      console.log(`Removing api ${apiConfig.apiId}`);
      await this.apigw.trigger.removeTrigger({
        serviceId,
        apiId: apiConfig.apiId,
      });

      try {
        await this.apigw.client.DeleteApi({
          ApiId: apiConfig.apiId,
          ServiceId: serviceId,
        });
      } catch (err) {
        // FIMXE: 有时候删除会有
        console.warn(err);
      }
    }
  }

  /**
   * 创建或更新 API 网关 API
   *
   * @param inputs 创建或更新 API 网关 API 输入
   * @returns 完成后的输出
   */
  async createOrUpdateApi(inputs: CreateOrUpdateApiInputs) {
    const { serviceId, endpoint, environment, created } = inputs;
    // compatibility for secret auth config depends on auth & usagePlan
    const authType = endpoint.auth ? 'SECRET' : endpoint.authType || 'NONE';
    const businessType = endpoint.businessType || 'NORMAL';
    const output: ApiDeployerOutputs = {
      path: endpoint.path,
      method: endpoint.method,
      apiName: endpoint.apiName || 'index',
      created: true,
      authType: authType,
      businessType: businessType,
      isBase64Encoded: endpoint.isBase64Encoded === true,
    };
    if (endpoint.authRelationApiId) {
      output.authRelationApiId = endpoint.authRelationApiId;
    }

    const apiInputs: Omit<CamelCasedProps<ModifyApiRequest>, 'apiId'> = {
      protocol: endpoint.protocol ?? 'HTTP',
      serviceTimeout: endpoint.serviceTimeout ?? 15,
      responseType: endpoint.responseType ?? 'HTML',
      apiName: endpoint.apiName ?? 'index',
      apiBusinessType: endpoint.businessType ?? 'NORMAL',
      serviceType: endpoint.serviceType ?? 'SCF',
      serviceId: serviceId,
      apiDesc: endpoint.description,
      apiType: 'NORMAL',
      authType: authType,
      requestConfig: {
        path: endpoint.path,
        method: endpoint.method,
      },
      enableCORS: endpoint.enableCORS === true,
      isBase64Encoded: endpoint.isBase64Encoded === true,
      oauthConfig: endpoint.oauthConfig,
      authRelationApiId: endpoint.authRelationApiId,
    };

    marshalApiInput(endpoint, apiInputs);

    let apiDetail: ApiInfo | undefined = undefined;

    if (endpoint.apiId) {
      apiDetail = await this.getApiById({ serviceId, apiId: endpoint.apiId });
    }

    if (!apiDetail) {
      apiDetail = await this.getApiByPathAndMethod({
        serviceId,
        path: endpoint.path,
        method: endpoint.method,
      });
    }

    if (apiDetail) {
      console.log(
        `Api method ${endpoint.method}, path ${endpoint.path} already exist as ${apiDetail.ApiId}`,
      );
      endpoint.apiId = apiDetail.ApiId;

      if (endpoint.isBase64Encoded && endpoint.isBase64Trigger) {
        apiInputs.isBase64Trigger = endpoint.isBase64Trigger;
        apiInputs.base64EncodedTriggerRules = endpoint.base64EncodedTriggerRules;
      }

      const modifyReq: ModifyApiRequest = {
        ...pascalCaseProperty(apiInputs),
        ApiId: endpoint.apiId,
        EnableCORS: apiInputs.enableCORS,
      };

      await this.apigw.client.ModifyApi(cleanReq(modifyReq));

      output.apiId = endpoint.apiId;
      output.created = !!created;
      output.internalDomain = apiDetail.InternalDomain || '';
      console.log(`Api ${output.apiId} updated`);
    } else {
      const createInputs: CreateApiRequest = {
        ...pascalCaseProperty(apiInputs),
        Protocol: endpoint.protocol ?? 'HTTP',
        ServiceTimeout: endpoint.serviceTimeout ?? 15,
        ResponseType: endpoint.responseType ?? 'HTML',
        ApiName: endpoint.apiName ?? 'index',
        ApiBusinessType: endpoint.businessType ?? 'NORMAL',
        EnableCORS: apiInputs.enableCORS,
      };
      delete (createInputs as any).EnableCors;
      const createRes = await this.apigw.client.CreateApi(removeUndefined(createInputs));
      const { ApiId } = createRes.Result!;
      output.apiId = ApiId;
      output.created = true;

      console.log(`API ${ApiId} created.`);
      const describeRes = await this.apigw.client.DescribeApi({
        ServiceId: serviceId,
        ApiId: output.apiId,
      });
      apiDetail = describeRes.Result;
      output.internalDomain = apiDetail?.InternalDomain ?? '';

      if (endpoint.isBase64Encoded && endpoint.isBase64Trigger) {
        apiInputs.isBase64Trigger = endpoint.isBase64Trigger;
        apiInputs.base64EncodedTriggerRules = endpoint.base64EncodedTriggerRules;
      }

      const modifyReq: ModifyApiRequest = {
        ApiId: ApiId,
        ...pascalCaseProperty(apiInputs),
        EnableCORS: apiInputs.enableCORS,
      };

      await this.apigw.client.ModifyApi(cleanReq(modifyReq));
    }

    output.apiName = apiInputs.apiName;

    if (endpoint.usagePlan) {
      const usagePlan = await this.apigw.usagePlan.bindUsagePlan({
        apiId: output.apiId,
        serviceId,
        environment,
        usagePlanConfig: endpoint.usagePlan,
        authConfig: endpoint.auth,
      });

      output.usagePlan = usagePlan;
    }

    return output;
  }

  /**
   * 通过服务 ID 和 API ID 获取 API 网关 API 信息
   *
   * @param inputs 输入
   * @param inputs.serviceId API 网关服务 ID
   * @param inputs.apiId API 网关 API ID
   * @returns API 详情
   */
  async getApiById(inputs: {
    /** API 网关服务 ID */
    serviceId: string;
    /** API 网关 API ID */
    apiId: string;
  }) {
    const { serviceId, apiId } = inputs;
    const res = await this.apigw.client.DescribeApi({
      ServiceId: serviceId,
      ApiId: apiId,
    });
    return res.Result;
  }

  /**
   * 根据路径和方法获取 API 网关 API 信息
   *
   * @param inputs 输入参数
   * @param inputs.serviceId API 网关服务 ID
   * @param inputs.path API 网关 API 接受路径
   * @param inputs.method API 网关 API 接受方法
   * @returns API 网关 API 详情
   */
  async getApiByPathAndMethod(inputs: {
    /** API 网关 serviceId */
    serviceId: string;
    /** API 路径 */
    path: string;
    /** API 网关 serviceId */
    method: string;
  }) {
    const { serviceId, path, method } = inputs;
    const res = await this.apigw.client.DescribeApisStatus({
      ServiceId: serviceId,
      Offset: 0,
      Limit: 100,
      Filters: [{ Name: 'ApiPath', Values: [path] }],
    });
    const { ApiIdStatusSet } = res.Result!;

    let apiStatus: DesApisStatus | undefined = undefined;

    if (ApiIdStatusSet) {
      for (const item of ApiIdStatusSet) {
        if (item.Path === path && item.Method.toLowerCase() === method.toLowerCase()) {
          apiStatus = item;
        }
      }
    }

    if (apiStatus) {
      const describeRes = await this.apigw.client.DescribeApi({
        ServiceId: serviceId,
        ApiId: apiStatus.ApiId,
      });

      return describeRes.Result;
    }
  }
}

==> apigw-1.1.0.tgz: package/src/api.ts <==
import {
  CamelCasedProps,
  pascalCaseProperty,
  removeUndefined,
} from '@tencent-serverless-framework/common';
import {
  ApiInfo,
  CreateApiRequest,
  DesApisStatus,
  ModifyApiRequest,
} from 'tencentcloud-sdk-nodejs/tencentcloud/services/apigateway/v20180808/apigateway_models';
import { Apigw } from '.';
import { marshalApiInput } from './formatter';
import {
  ApiDeployerInputs,
  ApiDeployerOutputs,
  ApigwApiRemoverInputs,
  CreateOrUpdateApiInputs,
} from './interface';
import { getLogger, printToConsole } from '@jsonlog/logger';

const logger = getLogger({
  name: 'APIGW - API',
  middlewares: [printToConsole],
});

/**
 * 清除请求中的无关参数
 *
 * @param req 请求
 * @returns 清除后的请求
 */
function cleanReq(req: any) {
  delete req.EnableCors;
  delete req.Created;
  return req;
}

/** API 网关组件 API 子模块 */
export class Api {
  /** API 网关组件 */
  apigw: Apigw;
  /**
   * 构造函数
   *
   * @param apigw API 网关组件（依赖注入）
   */
  constructor(apigw: Apigw) {
    this.apigw = apigw;
  }

  /**
   * 部署 API 网关 API列表
   *
   * @param inputs 部署 API 网关输入
   * @returns 部署 API 网关输出
   */
  async apiDeployer(inputs: ApiDeployerInputs): Promise<ApiDeployerOutputs> {
    const { serviceId, environment, apiList = [], oldList, apiConfig, isOauthApi } = inputs;
    // if exist in state list, set created to be true
    const [exist] = oldList.filter(
      (item) =>
        item.method.toLowerCase() === apiConfig.method.toLowerCase() &&
        item.path === apiConfig.path,
    );

    if (exist) {
      apiConfig.apiId = exist.apiId;
      apiConfig.created = exist.created;

      if (isOauthApi) {
        apiConfig.authRelationApiId = exist.authRelationApiId;
      }
    }
    if (isOauthApi && !apiConfig.authRelationApiId) {
      // find reletive oauth api
      const { authRelationApi } = apiConfig;
      if (authRelationApi) {
        const [relativeApi] = apiList.filter(
          (item) =>
            item.method.toLowerCase() === authRelationApi.method.toLowerCase() &&
            item.path === authRelationApi.path,
        );
        if (relativeApi) {
          apiConfig.authRelationApiId = relativeApi.apiId;
        }
      }
    }

    const curApi = await this.createOrUpdateApi({
      serviceId,
      environment,
      endpoint: apiConfig,
      created: exist && exist.created,
    });

    logger.info({ msg: `部署 API ${curApi.apiName} 成功` });
    return curApi;
  }

  /**
   * 移除 API 网关 API
   *
   * @param inputs 移除 API 网关 API输入参数
   */
  async apiRemover(inputs: ApigwApiRemoverInputs) {
    const { apiConfig, serviceId, environment } = inputs;
    // 1. remove usage plan
    if (apiConfig.usagePlan) {
      await this.apigw.usagePlan.removeOrUnbindUsagePlan({
        serviceId,
        environment,
        apiId: apiConfig.apiId,
        usagePlan: apiConfig.usagePlan,
      });
    }

    // 2. delete only apis created by serverless framework
    if (apiConfig.apiId && apiConfig.created === true) {
      logger.info({ msg: `移除 API ${apiConfig.apiId} 成功` });
      await this.apigw.trigger.removeTrigger({
        serviceId,
        apiId: apiConfig.apiId,
      });

      try {
        await this.apigw.client.DeleteApi({
          ApiId: apiConfig.apiId,
          ServiceId: serviceId,
        });
      } catch (err) {
        // FIMXE: 有时候删除会有
        logger.warn({ msg: `发生错误 ${err}` });
      }
    }
  }

  /**
   * 创建或更新 API 网关 API
   *
   * @param inputs 创建或更新 API 网关 API 输入
   * @returns 完成后的输出
   */
  async createOrUpdateApi(inputs: CreateOrUpdateApiInputs) {
    const { serviceId, endpoint, environment, created } = inputs;
    // compatibility for secret auth config depends on auth & usagePlan
    const authType = endpoint.auth ? 'SECRET' : endpoint.authType || 'NONE';
    const businessType = endpoint.businessType || 'NORMAL';
    const output: ApiDeployerOutputs = {
      path: endpoint.path,
      method: endpoint.method,
      apiName: endpoint.apiName || 'index',
      created: true,
      authType: authType,
      businessType: businessType,
      isBase64Encoded: endpoint.isBase64Encoded === true,
    };
    if (endpoint.authRelationApiId) {
      output.authRelationApiId = endpoint.authRelationApiId;
    }

    const apiInputs: Omit<CamelCasedProps<ModifyApiRequest>, 'apiId'> = {
      protocol: endpoint.protocol ?? 'HTTP',
      serviceTimeout: endpoint.serviceTimeout ?? 15,
      responseType: endpoint.responseType ?? 'HTML',
      apiName: endpoint.apiName ?? 'index',
      apiBusinessType: endpoint.businessType ?? 'NORMAL',
      serviceType: endpoint.serviceType ?? 'SCF',
      serviceId: serviceId,
      apiDesc: endpoint.description,
      apiType: 'NORMAL',
      authType: authType,
      requestConfig: {
        path: endpoint.path,
        method: endpoint.method,
      },
      enableCORS: endpoint.enableCORS === true,
      isBase64Encoded: endpoint.isBase64Encoded === true,
      oauthConfig: endpoint.oauthConfig,
      authRelationApiId: endpoint.authRelationApiId,
    };

    marshalApiInput(endpoint, apiInputs);

    let apiDetail: ApiInfo | undefined = undefined;

    if (endpoint.apiId) {
      apiDetail = await this.getApiById({ serviceId, apiId: endpoint.apiId });
    }

    if (!apiDetail) {
      apiDetail = await this.getApiByPathAndMethod({
        serviceId,
        path: endpoint.path,
        method: endpoint.method,
      });
    }

    if (apiDetail) {
      logger.info({
        msg: `API 方法=${endpoint.method}, 路径=${endpoint.path} 已经存在，ID 为 ${apiDetail.ApiId}`,
      });
      endpoint.apiId = apiDetail.ApiId;

      if (endpoint.isBase64Encoded && endpoint.isBase64Trigger) {
        apiInputs.isBase64Trigger = endpoint.isBase64Trigger;
        apiInputs.base64EncodedTriggerRules = endpoint.base64EncodedTriggerRules;
      }

      const modifyReq: ModifyApiRequest = {
        ...pascalCaseProperty(apiInputs),
        ApiId: endpoint.apiId,
        EnableCORS: apiInputs.enableCORS,
      };

      await this.apigw.client.ModifyApi(cleanReq(modifyReq));

      output.apiId = endpoint.apiId;
      output.created = !!created;
      output.internalDomain = apiDetail.InternalDomain || '';
      logger.info({ msg: `API ${output.apiId} 已经更新` });
    } else {
      const createInputs: CreateApiRequest = {
        ...pascalCaseProperty(apiInputs),
        Protocol: endpoint.protocol ?? 'HTTP',
        ServiceTimeout: endpoint.serviceTimeout ?? 15,
        ResponseType: endpoint.responseType ?? 'HTML',
        ApiName: endpoint.apiName ?? 'index',
        ApiBusinessType: endpoint.businessType ?? 'NORMAL',
        EnableCORS: apiInputs.enableCORS,
      };
      delete (createInputs as any).EnableCors;
      const createRes = await this.apigw.client.CreateApi(removeUndefined(createInputs));
      const { ApiId } = createRes.Result!;
      output.apiId = ApiId;
      output.created = true;

      logger.info({ msg: `API ${ApiId} 已经创建` });
      const describeRes = await this.apigw.client.DescribeApi({
        ServiceId: serviceId,
        ApiId: output.apiId,
      });
      apiDetail = describeRes.Result;
      output.internalDomain = apiDetail?.InternalDomain ?? '';

      if (endpoint.isBase64Encoded && endpoint.isBase64Trigger) {
        apiInputs.isBase64Trigger = endpoint.isBase64Trigger;
        apiInputs.base64EncodedTriggerRules = endpoint.base64EncodedTriggerRules;
      }

      const modifyReq: ModifyApiRequest = {
        ApiId: ApiId,
        ...pascalCaseProperty(apiInputs),
        EnableCORS: apiInputs.enableCORS,
      };

      await this.apigw.client.ModifyApi(cleanReq(modifyReq));
    }

    output.apiName = apiInputs.apiName;

    if (endpoint.usagePlan) {
      const usagePlan = await this.apigw.usagePlan.bindUsagePlan({
        apiId: output.apiId,
        serviceId,
        environment,
        usagePlanConfig: endpoint.usagePlan,
        authConfig: endpoint.auth,
      });

      output.usagePlan = usagePlan;
    }

    return output;
  }

  /**
   * 通过服务 ID 和 API ID 获取 API 网关 API 信息
   *
   * @param inputs 输入
   * @param inputs.serviceId API 网关服务 ID
   * @param inputs.apiId API 网关 API ID
   * @returns API 详情
   */
  async getApiById(inputs: {
    /** API 网关服务 ID */
    serviceId: string;
    /** API 网关 API ID */
    apiId: string;
  }) {
    const { serviceId, apiId } = inputs;
    const res = await this.apigw.client.DescribeApi({
      ServiceId: serviceId,
      ApiId: apiId,
    });
    return res.Result;
  }

  /**
   * 根据路径和方法获取 API 网关 API 信息
   *
   * @param inputs 输入参数
   * @param inputs.serviceId API 网关服务 ID
   * @param inputs.path API 网关 API 接受路径
   * @param inputs.method API 网关 API 接受方法
   * @returns API 网关 API 详情
   */
  async getApiByPathAndMethod(inputs: {
    /** API 网关 serviceId */
    serviceId: string;
    /** API 路径 */
    path: string;
    /** API 网关 serviceId */
    method: string;
  }) {
    const { serviceId, path, method } = inputs;
    const res = await this.apigw.client.DescribeApisStatus({
      ServiceId: serviceId,
      Offset: 0,
      Limit: 100,
      Filters: [{ Name: 'ApiPath', Values: [path] }],
    });
    const { ApiIdStatusSet } = res.Result!;

    let apiStatus: DesApisStatus | undefined = undefined;

    if (ApiIdStatusSet) {
      for (const item of ApiIdStatusSet) {
        if (item.Path === path && item.Method.toLowerCase() === method.toLowerCase()) {
          apiStatus = item;
        }
      }
    }

    if (apiStatus) {
      const describeRes = await this.apigw.client.DescribeApi({
        ServiceId: serviceId,
        ApiId: apiStatus.ApiId,
      });

      return describeRes.Result;
    }
  }
}
